#include <ddraw.h>
#include <ddutil.h>
#include <assert.h>

extern "C" {

extern LPDIRECTDRAW GetDirectDrawInterface (int iWndIndex);
extern LPDIRECTDRAWSURFACE GetBackBuffer (int iWndIndex);
extern printf (const char *, ...);

LPDIRECTDRAW lpDD = 0;
LPDIRECTDRAWSURFACE lpBack = 0;
LPDIRECTDRAWSURFACE lpFront = 0;

LPDIRECTDRAWSURFACE
LoadACMBitmap (char *szBitmap, int * width, int * height)
{
	HBITMAP	hbm;
	DDSURFACEDESC ddsd;
	LPDIRECTDRAWSURFACE lpDDS;
	HRESULT	ddrval;
	BITMAP              bm;

	// Lazy initialization

	if (!lpDD) {
		lpDD = GetDirectDrawInterface ( 0 );
	}

	// Load bitmap and get dimensions

	hbm = (HBITMAP) LoadImage (GetModuleHandle(NULL), szBitmap,
		IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);

	if (hbm == NULL) {
		return NULL;
	}

	GetObject(hbm, sizeof(bm), &bm);    // get size of bitmap
    *width = bm.bmWidth;
    *height = bm.bmHeight;

	// Create an off-screen bitmap.
	ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = *width;
    ddsd.dwHeight = *height;
	ddrval = lpDD->CreateSurface(&ddsd, &lpDDS, NULL);
	switch (ddrval) {
	case DDERR_INCOMPATIBLEPRIMARY:
		printf ("INCOMPATIBLEPRIMARY\n");
		break;
	case DDERR_INVALIDCAPS:
		printf ("INVALIDCAPS\n");
		break;
	case DDERR_INVALIDOBJECT:
		printf ("INVALIDOBJECT\n");
		break;
	case DDERR_INVALIDPARAMS:
		printf ("INVALIDPARAMS\n");
		break;
	case DDERR_INVALIDPIXELFORMAT:
	case DDERR_NOALPHAHW:
	case DDERR_NOCOOPERATIVELEVELSET:
	case DDERR_NODIRECTDRAWHW:
	case DDERR_NOEMULATION:
	case DDERR_NOEXCLUSIVEMODE:
	case DDERR_NOFLIPHW:
	case DDERR_NOMIPMAPHW:
	case DDERR_NOZBUFFERHW:
	case DDERR_OUTOFMEMORY:
	case DDERR_OUTOFVIDEOMEMORY:
		printf ("other\n");
		break;
	case DDERR_PRIMARYSURFACEALREADYEXISTS:
		printf ("PRIMARYSURFACEALREADYEXISTS\n");
		break;
	case DDERR_UNSUPPORTEDMODE:
		printf ("UNSUPPORTEDMODE\n");
		break;
	}


	if(ddrval != DD_OK) {
		*width = *height = 0;
		return NULL;
	}

	DDCopyBitmap(lpDDS, hbm, 0, 0, *width, *height);

	DeleteObject(hbm);

	return lpDDS;
}

void
PutACMBitmap (LPDIRECTDRAWSURFACE bm, int w, int h, int x, int y, int len)
{
	RECT        rcRect;
	HRESULT	ddrval;

	if (!lpBack) {
		lpBack = GetBackBuffer (0);
	}

		rcRect.left = 0;
		rcRect.top = 0;
		rcRect.right = w-1;
		rcRect.bottom = h-1;
		while(1) {
			ddrval = lpBack->BltFast(x, y, bm, &rcRect, FALSE);
			if(ddrval == DD_OK) {
				break;
			}
		}
}


}

