/*
 *   DIS/x : An implementation of the IEEE 1278.1 protocol
 *
 *   Copyright (C) 1997, Riley Rainey (rrainey@ix.netcom.com)
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of either:
 *
 *   a) the GNU Library General Public License as published by the Free
 *   Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.  A description of the terms and conditions
 *   of the GLPL may be found in the "COPYING.LIB" file.
 *
 *   b) the "Artistic License" which comes with this Kit.  Information
 *   about this license may be found in the "Artistic" file.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License or the Artistic License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Information describing how to contact the author can be found in the
 *   README file.
 */

#ifndef _dis_h
#define _dis_h

#include <dis/disenum.h>
#include <dis/disenum2.h>
#include <dis/disenum3.h>
#include <rpc/types.h>
#include <dis/disp.h>
#include <dis/simx.h>
#include <dis/datum.h>
#include <sys/types.h>
#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <crtdbg.h>
#else
#include <sys/time.h>
#include <netinet/in.h>
#endif
#include <math.h>

typedef struct _DISDestinationAddress {
	struct sockaddr_in	addr;
	int			type;	/* 0=broadcast; 1=other */
	} DISDestinationAddress;

typedef struct _DISTransceiver {
	int			s;
	DISDestinationAddress	dest[32];
	int			num_dest;
	} DISTransceiver;

typedef enum {
	DISResultOK = 0,
	DISResultError = 1,
	DISResultNoMemory = 2
	} DISResult;

typedef struct _DISxApplicationInfo {
	DISTransceiver		*xcvr;
	dis_pdu_header		hdr;
	dis_simulation_addr	id;
	u_short			last_event;
	u_short			last_entity;
	} DISxApplicationInfo;

extern DISxApplicationInfo *
	DISxInitializeApplication (unsigned int exercise_id,
		unsigned int site_id,
		unsigned int application_id);
extern int DISxReadPDU (DISxApplicationInfo *, dis_pdu *);
extern int DISxWritePDU (DISxApplicationInfo *, dis_pdu *);
extern void DISxGetSimulationAddress (DISxApplicationInfo *info, 
	dis_simulation_addr *p);
extern dis_entity_id * DISxIssueEntityID (DISxApplicationInfo *info,
	dis_entity_id *p);	
extern dis_event_id * DISxIssueEventID (DISxApplicationInfo *info,
	dis_event_id *p);

extern DISResult DISAddArticulationParm (dis_pdu *p,
										 dis_articulation_parm *parm,
										 int *parmID);	

extern int DISSetNBIOState (DISTransceiver *, int); 
extern DISTransceiver * DISOpenTransceiver (int port);
extern int DISReadPDU (DISTransceiver *, dis_pdu *);
extern int DISWritePDU (DISTransceiver *, dis_pdu *);

extern void DISFreePDUComponents (dis_pdu *);
extern void DISAddPDUSizes (dis_pdu *);

extern int DISGetRealTime (dis_time *result);
extern int DISGetTimestamp (dis_timestamp *result);
extern void DISTimeToTimeval (dis_time *in, struct timeval *out);
extern void DISTimestampToTimeval (dis_timestamp *in, struct timeval *out);

/*
 *  Round world stuff
 */

typedef struct _worldcoordinates {
	double	latitude;	/* [radians, north positive] */
	double  longitude;	/* [radians, east positive] */
	double  z;		/* above reference ellipsoid [meters] */
	}	WorldCoordinates;

typedef enum {
	LLM_DMS,		/* dd mm ss.s [EWNS] */
	LLM_DM,			/* dd mm [EWNS] */
	LLM_D,			/* dd [EWNS] */
	LLM_SIGNED_D
	} LatLongDisplayFormat;


#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif

#ifndef DEGtoRAD
#define DEGtoRAD(x)	((x) * M_PI / 180.0)
#define RADtoDEG(x)	((x) * 180.0 / M_PI)
#endif

#define SECtoRAD(x)	((x) * M_PI / (180.0 * 3600.0))

#define WGS84_MAJOR	6378137.0	/* meters */
#define WGS84_MINOR	6356752.3142	/* meters */
#define WGS84_ECC	0.081819190928906199466 /* eccentricity */
#define WGS84_ECC_SQR	0.006694380004260806515 /* eccentricity squared */

extern void
DISUpdateWorldCoordinates(WorldCoordinates * p, double sin_course, double cos_course, double d);
extern char *
DISLatitudeToString(char *s, double la, LatLongDisplayFormat mode);
extern char *
DISLongitudeToString(char *s, double lo, LatLongDisplayFormat mode);
extern void
DISGeocentricToWorldCoordinates(dis_world_coordinates * p, WorldCoordinates * w);
extern void
DISWorldCoordinatesToGeocentric(WorldCoordinates * w, dis_world_coordinates * p);
extern char *
DISStringToLatLong(char *s, WorldCoordinates * w);

#endif
