/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1994  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */
#include <math.h>
#include <stdio.h>

#include <client.h>

#include <flaps0.xbm>
#include <flaps1.xbm>
#include <flaps2.xbm>
#include <flaps3.xbm>
#include <gearDown.xbm>
#include <gearTran.xbm>
#include <gearUp.xbm>
#include <handleDn.xbm>
#include <handleUp.xbm>
#include <eng.xbm>

extern char *ItoA();
extern void DoTEWSDisplay PARAMS((craft*, viewer *));
extern void DoRadarDisplay PARAMS((craft *, viewer *));
extern void PlaceCraft();
extern void doScale(), doCompassScale();

#define lim	((double) 200 * NM)

VPoint hex [6] =	{
	{  1.00000,  0.00000, 0.0 },
	{  0.50000,  0.86603, 0.0 },
	{ -0.50000,  0.86603, 0.0 },
	{ -1.00000,  0.00000, 0.0 },
	{ -0.50000, -0.86603, 0.0 },
	{  0.50000, -0.86603, 0.0 }
	};

long polyCount;
VPolygon **poly;

static craft *sortList;

void insertCraft();

void
InitPixmaps (u)
viewer	*u;
{

    u->eng = XCreateBitmapFromData (u->dpy, u->win, eng_bits, eng_width,
	eng_height);
    u->flap[0] = XCreateBitmapFromData (u->dpy, u->win, flaps0_bits,
	flaps0_width, flaps0_height);
    u->flap[1] = XCreateBitmapFromData (u->dpy, u->win, flaps1_bits,
	flaps1_width, flaps1_height);
    u->flap[2] = XCreateBitmapFromData (u->dpy, u->win, flaps2_bits,
	flaps2_width, flaps2_height);
    u->flap[3] = XCreateBitmapFromData (u->dpy, u->win, flaps3_bits,
	flaps3_width, flaps3_height);
    u->handle[0] = XCreateBitmapFromData (u->dpy, u->win, handleUp_bits,
	handleUp_width, handleUp_height);
    u->handle[1] = XCreateBitmapFromData (u->dpy, u->win, handleDn_bits,
	handleDn_width, handleDn_height);
    u->gearLight[0] = XCreateBitmapFromData (u->dpy, u->win, gearUp_bits,
	gearUp_width, gearUp_height);
    u->gearLight[1] = XCreateBitmapFromData (u->dpy, u->win, gearTran_bits,
	gearTran_width, gearTran_height);
    u->gearLight[2] = XCreateBitmapFromData (u->dpy, u->win, gearDown_bits,
	gearDown_width, gearDown_height);

}

void
addGround(c, poly, count, color, z)
craft	*c;
VPolygon **poly;
long	*count;
VColor	*color;
double	z;
{
	VPoint	hex1[6];
	double	d;
	int	i, j, k;
	long	cnt = *count;


/*
 *  We should really be drawing all these hexagons if any of the viewers
 *  is in depth-cueued mode, not just the real player.
 */

	if (c->vl->v->flags & VPDepthCueing) {
	    k = _VDefaultWorkContext->depthCueSteps-1;
	    for (i=0; i<_VDefaultWorkContext->depthCueSteps; ++i, --k) {
	    	d = (i == 0) ? lim : (_VDefaultWorkContext->visTable[k]);
		for (j=0; j<6; ++j) {
			hex1[j].x = hex[j].x * d + c->Sg.x;
			hex1[j].y = hex[j].y * d + c->Sg.y;
			hex1[j].z = z;
		}
		poly[cnt] = VCreatePolygon (6, hex1, color);
		poly[cnt++]->assignedDepth = k;
	    }
	}

/*
 *  Otherwise draw just a single "ground" hexagon.
 */

	else {
		d = lim;
		for (j=0; j<6; ++j) {
			hex1[j].x = hex[j].x * d + c->Sg.x;
			hex1[j].y = hex[j].y * d + c->Sg.y;
			hex1[j].z = z;
		}
		poly[cnt++] = VCreatePolygon (6, hex1, color);
	}
	*count = cnt;
}

void
Render()
{

	register unsigned long curPixel;
	register craft	*c, *p;
	register int	i, j, k;
	register viewer *q;
	VPoint	 	vp, fwd, up;
	VPoint		horiz[4];
	int		mono;
	static ZInfo	z, panelz;
	short		bgIndex;

/*
 *  Build a vector of polygons for all objects in the scene.
 *
 *  This vector should be ordered from "most distant" to "closest" so that
 *  the final display will end up correct.  Rather than generalizing this
 *  to death, we'll use a few heuristics to get very close to what we need:
 *
 *  (0) Build a single polygon to represent the ground.
 *  (1) Objects on the surface (stbl) are collected first.
 *  (2) Planes and projectiles (ptbl and mtbl) are first sorted in descending
 *      order by their distance from the observer and then polygons are
 *	collected.
 */

	c = &ptbl[player_id];

/*
 *  Set up the eyespace transformation for this viewpoint
 */

	VTransform (&c->cinfo->viewPoint, &(c->trihedral), &vp);
	vp.x += c->Sg.x;
	vp.y += c->Sg.y;
	vp.z += c->Sg.z;

	VTransform (&c->viewDirection, &(c->trihedral), &fwd);
	fwd.x += vp.x;
	fwd.y += vp.y;
	fwd.z += vp.z;

	VTransform (&c->viewUp, &(c->trihedral), &up);
	up.x += vp.x;
	up.y += vp.y;
	up.z += vp.z;

	VGetEyeSpace (c->vl->v, vp, fwd, up);

	polyCount = 0;
	sortList = (craft *) NULL;

/*
 *  With our simple cloud model, we are either above, in, or below the clouds
 *
 *  What we display in the outside-cockpit view will depend of where we are.
 */

	bgIndex = 0;
	if (c->Sg.z < ctop) {

		addGround (c, poly, &polyCount, cloudColor, ctop);
		for (j=0; j<polyCount; ++j) {
		    VTransformPolygon (poly[j], &c->vl->v->eyeSpace);
	    	}

		for ((j=0, p=ptbl); j<MAXPLAYERS; (++j, ++p)) {
		    if (p->type != CT_FREE &&
			(p != c || c->flags & FL_CHASE_VIEW) &&
			p->Sg.z < ctop) {
				insertCraft (c, p);
			}
		}

		for ((j=0, p=mtbl); j<MAXPROJECTILES; (++j, ++p)) {
			if (p->type != CT_FREE && p->Sg.z < ctop) {
				insertCraft (c, p);
			}
		}
	}
	else if (c->Sg.z < cbase) {
		bgIndex = cloudColor->cIndex;
	}
	else {

	    addGround (c, poly, &polyCount, groundColor, 0.0);
	    for (j=0; j<polyCount; ++j) {
		VTransformPolygon (poly[j], &c->vl->v->eyeSpace);
	    }

	    for ((j=0, p=stbl); j<MAXSURFACE; (++j, ++p)) {
		if (p->type != CT_FREE)
		    insertCraft (c, p);
	    }

	    for ((j=0, p=ptbl); j<MAXPLAYERS; (++j, ++p)) {
		if (p->type != CT_FREE &&
			(p != c || c->flags & FL_CHASE_VIEW) &&
			p->Sg.z > cbase) {
		    insertCraft (c, p);
		}
	    }

	    for ((j=0, p=mtbl); j<MAXPROJECTILES; (++j, ++p)) {
		if (p->type != CT_FREE && p->Sg.z > cbase) {
		    insertCraft (c, p);
		}
	    }
	}

	for (p=sortList; p != (craft *) NULL; p=(craft *)p->next) {
		PlaceCraft (c->vl->v, c, p, poly, &polyCount);
	}

/*
 *  Clip all polygons
 */

        for (j=0; j<polyCount; ++j) {
            poly[j] = VClipSidedPolygon(c->vl->v, poly[j],
		c->vl->v->clipPoly);
	}

/*
 *  Fill in the sky color
 */

	q = c->vl;
	z.depth = MaxDepth;
	z.color = q->v->pixel[bgIndex];
	FillRectangle (q->v->w, 0, 0, q->width, q->height, &z);
	panelz.depth = MaxDepth;
	panelz.color = q->v->pixel[blackPixel];
	FillRectangle (q->v->w, q->rx, q->ry, 
		q->width - q->rx, q->radarWidth, &panelz);

/*
 *  Now the polygons ..
 */


	for (j=0; j<polyCount; ++j) {
                if (poly[j]) {
			VFillPolygon (q->v, q->win, q->gc, poly[j]);
                }
	}

/*  Draw Head-Up Display and instrument panel */

	doHUD(c, q);
	DoRadarDisplay (c, q);
	DoTEWSDisplay (c, q);		/* doRadar must be called first */
	doRPM(c, q);


/*  Expose the completed drawing  */

        VExposeBuffer (q->v, q->gc);

	doPanel (c,q);

	switch (q->redrew) {
	case 0:
		break;
	case 1:
		q->redrew = 0;
		break;
	case 2:
		q->redrew = 1;
		break;
	}

/*
 *  Release polygons
 */

        for (j=0; j<polyCount; ++j) {
            if (poly[j]) {
	        VDestroyPolygon (poly[j]);
	    }
	}

}

/*
 *  insertCraft :  Insert a craft into sortList in descending order.
 */

void
insertCraft (c, p)
craft *c, *p;
{

    double dist, offset;
    VPoint ds;
    craft  *q, *prev;

/*
 *  Here's a kludge for you:  to avoid polygon clipping, I'm going to 
 *  cheat and hack a way to get ground objects to display properly.
 *  if the Z coordinate of an object is zero (i.e. on ground objects),
 *  I'll add a huge offset to their distance values to force them to be
 *  plotted first -- and in roughly their correct drawing order.
 *
 */

    offset = (p->Sg.z == 0.0) ? 1000.0 * NM : 0.0;

    ds.x = p->Sg.x - c->Sg.x;
    ds.y = p->Sg.y - c->Sg.y;
    ds.z = p->Sg.z - c->Sg.z;
    p->order = dist = mag (ds) + offset;

    if (sortList != (craft *) NULL) {
        for ((q=sortList, prev=(craft *) NULL); q != (craft *) NULL;) {
	    if (q->order < dist) {
		p->next = (struct _craft *) q;
		if (prev == (craft *) NULL)
		    sortList = p;
		else
		    prev->next = (struct _craft *) p;
		break;
	    }
	    prev = q;
	    q = (craft *) q->next;
        }
        if (q == (craft *) NULL) {
	    prev->next = (struct _craft *) p;
	    p->next = (struct _craft *) NULL;
        }
    }
    else {
        sortList = p;
        p->next = (struct _craft *) NULL;
    }
}

int
doRPM (c, u)
craft *c;
viewer *u;{

	int	x, y, x0, y0, x1, y1, len, i, ffth, fftw, last;
	double  percent, p, sp, cp;
	char	s[32], s1[32];

	x = ENG_X * u->width / FS_WINDOW_WIDTH;
	y = ENG_Y * u->width / FS_WINDOW_WIDTH;
	len = eng_x_hot - 6;
	ffth = u->rfont->max_bounds.ascent + u->rfont->max_bounds.descent;
	fftw = u->rfont->max_bounds.width;

	if (isFunctioning (c, SYS_ENGINE1))
		percent = (double) c->rpm * 100.0;
	else
		percent = 0.0;

/*
 *  We only redraw the RPM gauge when we're pretty sure that the gauge moved.
 *  (forced redraws are triggered in panel.c).
 */

	sprintf (s, "%3d%s", (int) (c->rpm * 100.0 + 0.5),
		(c->flags & FL_AFTERBURNER) ? "AB" : "  ");
	if (strcmp (s, c->lastRPM) != 0) {

		XCopyPlane (u->dpy, u->eng, u->win, u->gauge_gc,
			0, 0, eng_width,eng_height,
			x - eng_x_hot, y - eng_y_hot, 1);

		if (percent < 60.0) {
			p = (60.0 - percent) / 120.0 * pi;
			sp = sin(p);
			cp = cos(p);
			x0 = x - (len >> 3) * cp;
			y0 = y + (len >> 3) * sp;
			x1 = x + len * cp;
			y1 = y - len * sp;
		}
		else {
			p = (percent - 60.0) / 40.0 * pi;
			sp = sin(p);
			cp = cos(p);
			x0 = x - (len >> 3) * cp;
			y0 = y - (len >> 3) * sp;
			x1 = x + len * cp;
			y1 = y + len * sp;
		}
		
		XDrawImageString (u->dpy, u->win,
			u->gauge_gc,
			x - eng_x_hot + 3 * fftw,
			y - eng_y_hot + ffth + ffth,
			s, strlen(s));
		strcpy (c->lastRPM, s);

		XDrawLine (u->dpy, u->win, u->gauge_gc, x0, y0, x1, y1);
	}

/*
 *  Print Fuel total and Fuel consumption rate
 */

	x = x - eng_x_hot;
	y = y + eng_x_hot + ffth + (ffth >> 1);

	sprintf (s, "%7s lbs Total", ItoA (((int) c->fuel)/10*10, s1));
	if (strcmp (s, c->lastTotal) != 0 || u->redrew > 0) {
		XDrawImageString (u->dpy, u->win, u->gauge_gc,
			x, y, s, strlen(s));
		strcpy (c->lastTotal, s);
	}

	c->curThrust = calcThrust (c);
	sprintf (s, "%7s lbs/hour",
		ItoA ((int) (((c->flags & FL_AFTERBURNER) ? 
			c->cinfo->spABFuelConsump : c->cinfo->spFuelConsump)
			* c->curThrust)/10*10, s1));
	if (strcmp (s, c->lastConsump) != 0 || u->redrew > 0) {
		XDrawImageString (u->dpy, u->win, u->gauge_gc,
			x, y+ffth+2, s, strlen(s));
		strcpy (c->lastConsump, s);
	}

/*
 *  Display Flap Setting
 */

	x = u->xCenter - (u->radarWidth + 1) / 2 - flaps0_width -
		(int) (20.0 * u->scaleFactor);
	y = FLAP_Y * u->width / FS_WINDOW_WIDTH;

/*
 *  We only adjust the flap indicator when we're pretty sure it moved
 */

	if ((last = (int) (c->flapSetting * 1000.0)) != u->lastFlap) {

		if (c->flapSetting > DEGtoRAD(29.0))
			i=3;
		else if (c->flapSetting > DEGtoRAD(19.0))
			i=2;
		else if (c->flapSetting > DEGtoRAD(9.0))
			i=1;
		else
			i=0;

		XCopyPlane (u->dpy, u->flap[i], u->win, u->gauge_gc,
			0, 0, flaps0_width, flaps0_height, x, y, 1);
	}

	sprintf (s, "Flaps: %d ", (int) RADtoDEG(c->curFlap));
	if (strcmp(s, c->lastFlap) || u->redrew > 0) {
		XDrawImageString (u->dpy, u->win, u->gauge_gc,
			x+flaps0_x_hot, y+flaps0_y_hot, s, strlen(s));
	}

	u->lastFlap = last;

	return 0;
}
