/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1997  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#include "pm.h"

int doGearLights(craft *c, viewer *u);

#ifdef WIN32

#include <ddraw.h>
#include "..\pc\resource.h"
#define handleDn_width 30

extern LPDIRECTDRAWSURFACE
LoadACMBitmap (char *szBitmap, int * width, int * height);
extern void
PutACMBitmap (LPDIRECTDRAWSURFACE bm, int w, int h, int x, int y, int len);

static LPDIRECTDRAWSURFACE gear_h[2];
int gh_width, gh_height;
static LPDIRECTDRAWSURFACE gear_light[3];
int gl_width, gl_height;
static LPDIRECTDRAWSURFACE flaps[4];
int flaps_width, flaps_height;

extern LPDIRECTDRAW GetDirectDrawInterface (int iWndIndex);
extern LPDIRECTDRAWSURFACE GetBackBuffer (int iWndIndex);

#else
#include "bitmap.h"
#endif

static struct {
    long mask;
    char *name;
} *pptr, panelVec[] = {

    {
	SYS_ENGINE1, "OIL PRES"
    },
    {
	SYS_HYD1, "HYD1 PRES"
    },
    {
	SYS_HYD2, "HYD2 PRES"
    },
    {
	SYS_GEN1, "GEN1 FAIL"
    },
    {
	SYS_GEN2, "GEN2 FAIL"
    },
    {
	SYS_FLAPS, "FLAP FAIL"
    },
    {
	SYS_SPEEDBRAKE, "SPBRK FAIL"
    },
    {
	SYS_RADAR, "RADAR FAIL"
    },
    {
	SYS_TEWS, "TEWS FAIL"
    },
    {
	SYS_HUD, " HUD FAIL"
    },
    {
	FLAG_LOWFUEL, " LOW FUEL"
    },
    {
	FLAG_SPEEDBRAKE, "SPD BRAKE"
    },
    {
	FLAG_WHEELBRAKE, "  BRAKES"
    },
    {
	0, (char *) 0
    }
};

#define panelRows	7
#define panelColumns	2
#define panelChars	10
#define lightMargin	((rftw) / 3)
#define panelWMargin	(rftw * 2)
#define panelHMargin	(rfth / 3)

long lastBits[MAXPLAYERS];
int rdraw[MAXPLAYERS];
int lastPos[MAXPLAYERS];
int lastGPos[MAXPLAYERS][3];

void
initPanel(c)
craft *c;
{

    rdraw[c->pIndex] = 0;
    lastPos[c->pIndex] = -1;
    lastGPos[c->pIndex][0] = lastGPos[c->pIndex][1] =
	lastGPos[c->pIndex][2] = -1;
    lastBits[c->pIndex] = SYS_NODAMAGE;

}

void
redrawPanel(c, u)
craft *c;
viewer *u;
{

    rdraw[c->pIndex] = 1;
    lastPos[c->pIndex] = -1;
    lastGPos[c->pIndex][0] = lastGPos[c->pIndex][1] =
	lastGPos[c->pIndex][2] = -1;
    lastBits[c->pIndex] = ~lastBits[c->pIndex];
    u->lastFlap = u->lastRPM = -1;
    u->redrew = 2;
}

static int rfth = 0, rftw = 0;

void
resizePanel(c, u)
craft *c;
viewer *u;
{
#ifdef WIN32
	if (rfth == 0) {
		LPDIRECTDRAWSURFACE lpBB = GetBackBuffer(0);
		TEXTMETRIC	tm;
		HFONT	hfont, hOldFont;
		HDC		hdc;
		HRESULT result;

		result = lpBB->lpVtbl->GetDC(lpBB, &hdc);
		if (result != DD_OK) {
			printf ("Background Surface error\n");
			exit (1);
		}

		hfont = GetStockObject (ANSI_FIXED_FONT);
		hOldFont = SelectObject (hdc, hfont);
		GetTextMetrics (hdc, &tm);
		rfth = tm.tmHeight;
		rftw = tm.tmAveCharWidth;
		result = lpBB->lpVtbl->ReleaseDC (lpBB, hdc);
		printf ("w/h = %d %d\n", rftw, rfth);
	}
#else

    rftw = u->rfont->max_bounds.width;

#endif

    u->panelx = (int) ((TEWS_X - 20) * u->scaleFactor + 0.5) -
	(u->TEWSSize + 1) / 2 -
	panelColumns * (panelChars * rftw + 2 * lightMargin +
			panelWMargin);
    u->panely = (int) (PANEL_Y * u->scaleFactor);
    redrawPanel(c, u);

}

void
doPanel(c, u)
craft *c;
viewer *u;
{
	int cellH, cellW;
    int xi, yi, x, y, i;
    long changeBits;
    XPoint topLeft;

#ifdef WIN32
	HFONT	hfont, hOldFont;
	HDC		hdc;
	HRESULT result;
	HBRUSH	hbrush;
	LPDIRECTDRAWSURFACE lpBB = GetBackBuffer(0);

	result = lpBB->lpVtbl->GetDC(lpBB, &hdc);
	if (result != DD_OK) {
		printf ("Background Surface error\n");
		exit (1);
	}

	hfont = GetStockObject (ANSI_FIXED_FONT);
	hOldFont = SelectObject (hdc, hfont);
	if (rfth == 0) {
		TEXTMETRIC	tm;
		GetTextMetrics (hdc, &tm);
		rfth = tm.tmHeight;
		rftw = tm.tmAveCharWidth;
	}

#else

    rfth = u->rfont->max_bounds.ascent + u->rfont->max_bounds.descent;
    rftw = u->rfont->max_bounds.width;

#endif

    cellH = rfth + 2 * lightMargin + panelHMargin + 1;
    cellW = rftw * panelChars + 2 * lightMargin + panelWMargin;

    changeBits = lastBits[c->pIndex] ^ c->damageBits;

    for (pptr = &panelVec[0], i = 0; pptr->mask != 0; ++pptr, ++i) {

		if (changeBits & pptr->mask) {
			xi = i / panelRows;
			yi = i % panelRows;
			x = u->panelx;
			x += xi * cellW + lightMargin;
			y = u->panely;
			y += yi * cellH + lightMargin;
			if ((c->damageBits & pptr->mask) == 0) {
				topLeft.x = x - lightMargin;
				topLeft.y = y - lightMargin;

#ifdef WIN32
				{
				RECT rect;
				hbrush = GetStockObject (WHITE_BRUSH);
				rect.left = topLeft.x;
				rect.top = topLeft.y;
				rect.right = rect.left + cellW - panelWMargin;
				rect.bottom = rect.top + cellH - panelHMargin;
				FrameRect (hdc, &rect, hbrush);
				SetTextColor(hdc, RGB(230, 230, 230));
				SetBkColor (hdc, RGB(0,0,0));
				TextOut (hdc, x, y, pptr->name, strlen (pptr->name));
				}
#else
				XDrawRectangle(u->dpy, u->win, u->gc,
			       topLeft.x, topLeft.y,
			       cellW - panelWMargin,
			       cellH - panelHMargin);
				XDrawImageString(u->dpy, u->win, u->gauge_gc,
					x, y+rfth, pptr->name, strlen(pptr->name));
#endif
			}
			else {
				topLeft.x = x - lightMargin;
				topLeft.y = y - lightMargin;
#ifdef WIN32
				{
				RECT rect;
				hbrush = GetStockObject (WHITE_BRUSH);
				rect.left = topLeft.x;
				rect.top = topLeft.y;
				rect.right = rect.left + cellW - panelWMargin;
				rect.bottom = rect.top + cellH - panelHMargin;
				FrameRect (hdc, &rect, hbrush);
				rect.top += lightMargin;
				rect.left += lightMargin;
				rect.right = rect.left + rftw * panelChars;
				rect.bottom = rect.top + rfth + lightMargin;
				hbrush = GetStockObject (BLACK_BRUSH);
				FillRect (hdc, &rect, hbrush);
				}
#else
				XDrawRectangle(u->dpy, u->win, u->gc,
			       topLeft.x, topLeft.y,
			       cellW - panelWMargin,
			       cellH - panelHMargin);
				/*y -= rfth;*/
				XClearArea(u->dpy, u->win, x, y,
					rftw * panelChars,
					rfth + lightMargin, False);
#endif
			}
		}
    }

    lastBits[c->pIndex] = c->damageBits;
    rdraw[c->pIndex] = 0;

#ifdef WIN32
	result = lpBB->lpVtbl->ReleaseDC (lpBB, hdc);
#endif

	 doGearLights(c, u);

}

int
doGearLights(c, u)
craft *c;
viewer *u;
{
	int x, y, x1, y1, pos;
    double geardown;

    pos = (c->flags & FL_GHANDLE_DN) ? 1 : 0;

	x = u->xCenter + (u->radarWidth + 1) / 2 + (int) (20.0 *
						   u->scaleFactor + 0.5);

    y = u->height + (int) ((VISOR_MARGIN + 80.0) * u->scaleFactor + 0.5);

#ifdef WIN32
	if (gear_h[0] == 0) {
		gear_h[0] = LoadACMBitmap (MAKEINTRESOURCE (IDB_GH_UP),
			&gh_width, &gh_height);
		gear_h[1] = LoadACMBitmap (MAKEINTRESOURCE (IDB_GH_DOWN),
			&gh_width, &gh_height);
		gear_light[0] = LoadACMBitmap (MAKEINTRESOURCE (IDB_GEAR_UP),
			&gl_width, &gl_height);
		gear_light[1] = LoadACMBitmap (MAKEINTRESOURCE (IDB_GEAR_TRANSIT),
			&gl_width, &gl_height);
		gear_light[2] = LoadACMBitmap (MAKEINTRESOURCE (IDB_GEAR_DOWN),
			&gl_width, &gl_height);
		flaps[0] = LoadACMBitmap (MAKEINTRESOURCE (IDB_FLAPS0),
			&flaps_width, &flaps_height);
		flaps[1] = LoadACMBitmap (MAKEINTRESOURCE (IDB_FLAPS1),
			&flaps_width, &flaps_height);
		flaps[0] = LoadACMBitmap (MAKEINTRESOURCE (IDB_FLAPS2),
			&flaps_width, &flaps_height);
		flaps[1] = LoadACMBitmap (MAKEINTRESOURCE (IDB_FLAPS3),
			&flaps_width, &flaps_height);
	}

	PutACMBitmap (gear_h[pos], gh_width, gh_height, x, y, 0);
#else

#ifdef notdef
    XSetForeground(u->dpy, u->gc, (u->v->flags & VPMono) ?
		   WhitePixel(u->v->dpy, u->v->screen) :
		   VConstantColor(u->v, whitePixel));

    XSetPlaneMask(u->dpy, u->gc, AllPlanes);
#endif

    if (pos != lastPos[c->pIndex]) {
	XCopyPlane(u->dpy, u->handle[pos], u->win, u->gauge_gc, 0, 0,
		   handleDn_width, handleDn_height, x, y, 1);
	lastPos[c->pIndex] = pos;
    }

#endif

    geardown = pi / 2.0;

    if (c->flags & FL_GEAR0_UP)
		pos = 0;
    else if (c->curGear[0] != geardown)
		pos = 1;
    else
		pos = 2;

    if (pos != lastGPos[c->pIndex][0]) {
		x1 = x + handleDn_width + 12 + (int) (8.0 * u->scaleFactor + 0.5);
		y1 = y + 10 + (int) (1.0 * u->scaleFactor + 0.5);
#ifdef WIN32
		PutACMBitmap (gear_light[pos], gl_width, gl_height, x1, y1, 0);
#else
		XCopyPlane(u->dpy, u->gearLight[pos], u->win, u->gauge_gc,
		   0, 0,
		   gearDown_width, gearDown_height, x1, y1, 1);
#endif
		lastGPos[c->pIndex][0] = pos;
    }
    if (c->flags & FL_GEAR1_UP)
		pos = 0;
    else if (c->curGear[1] != geardown)
		pos = 1;
    else
		pos = 2;

    if (pos != lastGPos[c->pIndex][1]) {
		x1 = x + handleDn_width;
		y1 = y + 50 + (int) (5.0 * u->scaleFactor + 0.5);
#ifdef WIN32
		PutACMBitmap (gear_light[pos], gl_width, gl_height, x1, y1, 0);
#else
		XCopyPlane(u->dpy, u->gearLight[pos], u->win, u->gauge_gc,
		   0, 0,
		   gearDown_width, gearDown_height, x1, y1, 1);
#endif
		lastGPos[c->pIndex][1] = pos;
    }

    if (c->flags & FL_GEAR2_UP)
		pos = 0;
    else if (c->curGear[2] != geardown)
		pos = 1;
    else
		pos = 2;

    if (pos != lastGPos[c->pIndex][2]) {
		x1 = x + handleDn_width + 25 + (int) (15.0 * u->scaleFactor + 0.5);
		y1 = y + 50 + (int) (5.0 * u->scaleFactor + 0.5);
#ifdef WIN32
		PutACMBitmap (gear_light[pos], gl_width, gl_height, x1, y1, 0);
#else
		XCopyPlane(u->dpy, u->gearLight[pos], u->win, u->gauge_gc,
		   0, 0,
		   gearDown_width, gearDown_height, x1, y1, 1);
#endif
		lastGPos[c->pIndex][2] = pos;
    }
    return 0;

}
