/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1997  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */
 
#include "pm.h"
#include "alarm.h"
#include <X11/Xutil.h>

#ifdef REAL_DELTA_T
#include <sys/time.h>

double lastTime = 0.0, sum = 0.0;
long frameCount=0;

#endif

#ifdef HAVE_DIS
long sendCount=0;
#endif

double gameEmptyTime = 0.0;


extern int flightCalculations();
extern int missileCalculations();
extern void doEvents(), doViews(), flapControl(), resupplyCheck();
extern void alarmCheck();
extern void blackBoxInput(), blackBoxOutput();

int cur = 0;
static int warned = 0;


#ifdef HAVE_DIS

#include "dis.h"


#define FT_TO_M(x)	(0.3048*(x))
#define M_TO_FT(x)	((x)/0.3048)

/*
 *  Convert transformation matrix values into an equivalent roll, pitch,
 *  and heading.
 */

void
matrixToEuler (VMatrix *mt, double *heading, double *pitch, double *roll)
{

	register double	i, j, k, m;

/*
 *  Compute the heading ...
 */

	i = mt->m[0][0];
	j = mt->m[1][0];
	k = mt->m[2][0];

	if (i == 0.0 && j == 0.0)
		*heading = 0.0;
	else if ((m = atan2 (j, i)) < 0.0)
		*heading = pi * 2.0 + m;
	else
		*heading = m;

/*
 *  and Pitch ...
 */

	*pitch = - asin (k);

/*
 *  and Roll ...
 */

	*roll = atan2 (mt->m[2][1], mt->m[2][2]);

}

/*
 *  Update DIS entity state corresponding to te given craft
 */

void
disBroadcast (p)
craft	*p;
{
	double location[3], velocity[3], linearAcceleration[3];
	double orientation[3], angularVelocity[3];
	VPoint tmp, tmp1;
	VMatrix XYZtoABC, NEDtoXYZ;
	double dt;
	int res;
  	static double base = -1;
	if (base < 0) base = curTime;

/*
 *  Well, this is a bit strange, but ACM's coordinate system for positions
 *  is meters in the Geocentric; for velocities are expressed as feet per
 *  second in the local NED [north-east-down] system.
 */

	location[0] = p->Sg.x;
	location[1] = p->Sg.y;
	location[2] = p->Sg.z;

	tmp.x = FEETtoMETERS(p->Cg.x);
	tmp.y = FEETtoMETERS(p->Cg.y);
	tmp.z = FEETtoMETERS(p->Cg.z);
	VReverseTransform_ (&tmp, &p->XYZtoNED, (VPoint *) &velocity[0]);

	VTransform (&p->linAcc, &p->trihedral, &tmp);
	tmp.x = FEETtoMETERS(tmp.x);
	tmp.y = FEETtoMETERS(tmp.y);
	tmp.z = FEETtoMETERS(tmp.z);
	VReverseTransform_ (&tmp, &p->XYZtoNED, (VPoint *) &linearAcceleration[0]);

	dt = curTime - p->disLastTime;
	p->disLastCg = p->Cg;

/*
 *  Derive ECI [Geocentric] heading, roll, pitch
 */

	VMatrixMultByRank (&p->XYZtoNED, &p->trihedral, &XYZtoABC, 3);
	matrixToEuler (&XYZtoABC,
	    &orientation[0], &orientation[1], &orientation[2]);

/*
 *  Derive angular velocity vector
 */

	tmp.x =  p->p; tmp.y =  p->q; tmp.z = - p->r;
	VReverseTransform_ (&tmp, &XYZtoABC, &angularVelocity[0]);

#ifdef notdef	
	printf ("X = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[0][0], XYZtoABC.m[0][1], XYZtoABC.m[0][2]);
	printf ("Y = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[1][0], XYZtoABC.m[1][1], XYZtoABC.m[1][2]);
	printf ("Z = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[2][0], XYZtoABC.m[2][1], XYZtoABC.m[2][2]);

	buildEulerMatrix (orientation[2], orientation[1], orientation[0],
		&XYZtoABC);
	printf ("---\nX = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[0][0], XYZtoABC.m[0][1], XYZtoABC.m[0][2]);
	printf ("Y = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[1][0], XYZtoABC.m[1][1], XYZtoABC.m[1][2]);
	printf ("Z = %7.4f %7.4f %7.4f\n",
		XYZtoABC.m[2][0], XYZtoABC.m[2][1], XYZtoABC.m[2][2]);

	printf ("ECI orientation [deg] (h,p,r) = %7.2f, %7.2f, %7.2f\n",
		RADtoDEG(orientation[0]), RADtoDEG(orientation[1]),
		RADtoDEG(orientation[2]));

	printf ("ECI angular velocities [deg/sec] (h,p,r) = %7.2f, %7.2f, %7.2f\n",
		RADtoDEG(angularVelocity[0]), RADtoDEG(angularVelocity[1]),
		RADtoDEG(angularVelocity[2]));
#endif

	res = dis_entityState (p->disId, location, velocity, linearAcceleration,
			       orientation, angularVelocity);
	if (res) {
	  sendCount++;
#ifdef notdef
	  printf("%6.2f: h=%8.2f, p=%8.2f, r=%8.2f, p=%7.2f, q=%7.2f, r=%7.2f\n",
		 curTime-base,
		 RADtoDEG(orientation[0]),
		 RADtoDEG(orientation[1]),
		 RADtoDEG(orientation[2]),
		 RADtoDEG(angularVelocity[0]),
		 RADtoDEG(angularVelocity[1]),
		 RADtoDEG(angularVelocity[2]));
	  printf("%.2f %.1f %.1f %.1f %.1f %.1f %.1f\n",
		 curTime-base,
		 RADtoDEG(orientation[0]),
		 RADtoDEG(orientation[1]),
		 RADtoDEG(orientation[2]),
		 RADtoDEG(angularVelocity[0]),
		 RADtoDEG(angularVelocity[1]),
		 RADtoDEG(angularVelocity[2]));
#endif
	}
}

void
disGetState (p)
craft	*p;
{
  	static double base = -1;
	double location[3];
	double velocity[3];
	double orientation[3];
	VPoint tmp;
	VMatrix XYZtoABC, NEDtoXYZ;
	int res;

	if (base < 0) base = curTime;

	res = dis_getEntityState (p->disId, location, velocity, orientation);
	/* if (res != 0) should not happen? */

	p->prevSg = p->Sg;
	p->Sg.x = location[0];
	p->Sg.y = location[1];
	p->Sg.z = location[2];

	DISGeocentricToWorldCoordinates
		( (dis_world_coordinates *) &p->Sg, &p->w );
	GenerateWorldToLocalMatrix (&p->w, &p->XYZtoNED);
	
	tmp.x = M_TO_FT(velocity[0]);
	tmp.y = M_TO_FT(velocity[1]);
	tmp.z = M_TO_FT(velocity[2]);
	VTransform_ (&tmp, &p->XYZtoNED, &p->Cg);

/*
 * TODO:  convert heading, pitch & roll from XYZ to NED.
 */

	p->curHeading = orientation[0];
	if (p->curHeading < 0.0) {
	    p->curHeading += 2*M_PI;
	}
	p->curPitch = orientation[1];
	p->curRoll = orientation[2];
	if (p->curRoll > M_PI) {
	    p->curRoll -= 2*M_PI;
	}

/*
 *  Aircraft trihedral is needed to calculate radar information; we
 *  probably do not need to update this every update frame.
 */

	buildEulerMatrix (p->curRoll, p->curPitch, p->curHeading,
		&XYZtoABC);
	transpose (&p->XYZtoNED, &NEDtoXYZ);
	VMatrixMultByRank (&NEDtoXYZ, &XYZtoABC, &p->trihedral, 3);

/*
 *  Now derive NED heading, pitch and roll from adjusted trihedral
 */
 
	euler (p);

#ifdef notdef
	printf("%d: h=%.0f, p=%.0f, r=%.0f\n",
	    p->disId,
	    RADtoDEG(p->curHeading),
	    RADtoDEG(p->curPitch),
	    RADtoDEG(p->curRoll));
	printf("%.2f %.1f %.1f %.1f\n",
	    curTime-base,
	    RADtoDEG(orientation[0]),
	    RADtoDEG(p->curPitch),
	    RADtoDEG(p->curRoll));
#endif
}

#endif



int
redraw()
{

	int	i, gameEmpty;
	craft	*p;
	struct timeval tp;
	struct timezone tzp;

	gettimeofday (&tp, &tzp);
	curTime = (double) tp.tv_sec + (double) tp.tv_usec / 1000000.0;

	if (lastTime == 0.0) {
		lastTime = curTime - 0.1;
	}

/*
 *  Check if "curTime" ever equals "lastTime", if so, warn that REAL_DELTA_T
 *  probably should not be used on this system.
 */

	if (curTime == lastTime) {
		curTime += 0.0001;
		if (warned) {
		    warned = 1;
		    fprintf (stderr,
"Hmm. We seem to have performed a complete inner loop in zero elapsed time.\n\
This seems suspicious.  Perhaps gettimeofday() has a relatively coarse\n\
granularity on this system.\n\
This server should probably be recompiled without REAL_DELTA_T defined.\n");
		}
	}

#ifdef REAL_DELTA_T
	deltaT = curTime - lastTime;

/*
 *  If the amount of real elapsed time is unreasonably long, then
 *  make it one update interval.
 */

	if (deltaT > (double)((UPDATE_INTERVAL / 1000000.0) * 4)) {
		deltaT = UPDATE_INTERVAL / 1000000.0;
	}
#else
	deltaT = UPDATE_INTERVAL / 1000000.0;
#endif /* REAL_DELTA_T */

	lastTime = curTime;
	halfDeltaTSquared = 0.5 * deltaT * deltaT;


#ifdef HAVE_DIS
	if (disInUse) {
	    if (disAbsoluteTime)
	        dis_setTimeAbsolute ();
	    else
		dis_setTime (curTime);
	    dis_receive ();
	}
#endif

	gameEmpty = 1;

	for ((i=0, p=ptbl); i<MAXPLAYERS; (++i, ++p)) {
		if ((p->type == CT_PLANE) && !(p->flags & FL_BLACK_BOX)) {
			gameEmpty = 0;
			doEvents (p);

			if (flightCalculations (p) == 1) {
				killPlayer (p);
			}
			doWeaponUpdate (p);
			flapControl (p);
#ifdef HAVE_DIS	  
			if (disInUse)
			    disBroadcast (p);
#endif	      
		}
	}

	for ((i=0, p=ptbl); i<MAXPLAYERS; (++i, ++p)) {
		if ((p->type == CT_DRONE) && !(p->flags & FL_BLACK_BOX)) {

			if (droneCalculations (p) == 1) {
				killPlayer (p);
			}

/*
 *  Calling flightCalculations makes drones adhere to the flight model.
 */
			if (flightCalculations (p) == 1) {
				killPlayer (p);
			}
			doWeaponUpdate (p);
			flapControl (p);
#ifdef HAVE_DIS	  
			if (disInUse)
			    disBroadcast (p);
#endif	      
		}
	}

#ifdef HAVE_DIS
	if (disInUse) {
	  for ((i=0, p=ptbl); i<MAXPLAYERS; (++i, ++p)) {
	      if (p->type == CT_DIS_PLANE)
	          disGetState (p);
	  }
	  for ((i=0, p=mtbl); i<MAXPROJECTILES; (++i, ++p)) {
	      if (p->type == CT_DIS_MUNITION)
	          disGetState (p);
	  }
	}
#endif	

	for ((i=0, p=ctbl); i<MAXPLAYERS; (++i, ++p)) {
	  if (p->type == CT_FREE) continue;
	  doEvents(p);
	}

	blackBoxInput();

	for ((i=0, p=mtbl); i<MAXPROJECTILES; (++i, ++p)) {
		if (p->type == CT_MISSILE) {
			if (missileCalculations (p) == 1) {
				killMissile (p, (craft *) NULL);
			}
#ifdef HAVE_DIS	  
			else if (disInUse) {
			    disBroadcast (p);
			}
#endif	
		}
		else if (p->type == CT_CANNON || p->type == CT_DIS_CANNON) {
			if (cannonCalculations (p) == 1) {
				killMissile (p, (craft *) NULL);
			}
		}
		else if (p->type == CT_EXPLOSION) {
			-- (p->flameDuration);
			if ((-- p->duration) <= 0)
				p->type = CT_FREE;
		}
	}

	lookForImpacts ();

#ifdef REAL_DELTA_T
	doViews();
#else
	if (cur++ % REDRAW_EVERY == 0)
		doViews ();
#endif

	blackBoxOutput();

	alarmCheck (deltaT);

#ifdef WATCH_FRAME_RATE
	sum += deltaT;
	frameCount++;
	if (frameCount % 100 == 0) {
		printf ("rate is %lf frames/second\n", (double)frameCount/sum );
#ifdef HAVE_DIS
		if (disInUse)
		  printf ("DIS send rate is %lf packets/second\n",
			  (double)sendCount/sum);
		sendCount = 0;
#endif	
		sum = deltaT;
		frameCount = 1;
	}
#endif	


	if (gameEmpty) {
		if ((gameEmptyTime += deltaT) > MAX_GAME_IDLE_SECONDS) {
			exit (0);
		}
	}
	else {
		gameEmptyTime = 0.0;
	}

	return 0;

}
